@@ -25,14 +25,14 @@
 #include <QtGui>
 #include <QtWidgets>
 
-#include <stdio.h>
+#include <cstdio>
 
 namespace OCC {
 
 static const char updateAvailableC[] = "Updater/updateAvailable";
 static const char updateTargetVersionC[] = "Updater/updateTargetVersion";
+static const char updateTargetVersionStringC[] = "Updater/updateTargetVersionString";
 static const char seenVersionC[] = "Updater/seenVersion";
-static const char autoUpdateFailedVersionC[] = "Updater/autoUpdateFailedVersion";
 static const char autoUpdateAttemptedC[] = "Updater/autoUpdateAttempted";
 
 
@@ -43,7 +43,7 @@ UpdaterScheduler::UpdaterScheduler(QObject *parent)
         this, &UpdaterScheduler::slotTimerFired);
 
     // Note: the sparkle-updater is not an OCUpdater
-    if (OCUpdater *updater = qobject_cast<OCUpdater *>(Updater::instance())) {
+    if (auto *updater = qobject_cast<OCUpdater *>(Updater::instance())) {
         connect(updater, &OCUpdater::newUpdateAvailable,
             this, &UpdaterScheduler::updaterAnnouncement);
         connect(updater, &OCUpdater::requestRestart, this, &UpdaterScheduler::requestRestart);
@@ -92,6 +92,11 @@ OCUpdater::OCUpdater(const QUrl &url)
 {
 }
 
+void OCUpdater::setUpdateUrl(const QUrl &url)
+{
+    _updateUrl = url;
+}
+
 bool OCUpdater::performUpdate()
 {
     ConfigFile cfg;
@@ -99,15 +104,20 @@ bool OCUpdater::performUpdate()
     QString updateFile = settings.value(updateAvailableC).toString();
     if (!updateFile.isEmpty() && QFile(updateFile).exists()
         && !updateSucceeded() /* Someone might have run the updater manually between restarts */) {
-        const QString name = Theme::instance()->appNameGUI();
-        if (QMessageBox::information(nullptr, tr("New %1 Update Ready").arg(name),
-                tr("A new update for %1 is about to be installed. The updater may ask\n"
-                   "for additional privileges during the process.")
-                    .arg(name),
-                QMessageBox::Ok)) {
+        const auto messageBoxStartInstaller = new QMessageBox(QMessageBox::Information,
+            tr("New %1 update ready").arg(Theme::instance()->appNameGUI()),
+            tr("A new update for %1 is about to be installed. The updater may ask "
+               "for additional privileges during the process. Your computer may reboot to complete the installation.")
+                .arg(Theme::instance()->appNameGUI()),
+            QMessageBox::Ok,
+            nullptr);
+
+        messageBoxStartInstaller->setAttribute(Qt::WA_DeleteOnClose);
+
+        connect(messageBoxStartInstaller, &QMessageBox::finished, this, [this] {
             slotStartInstaller();
-            return true;
-        }
+        });
+        messageBoxStartInstaller->open();
     }
     return false;
 }
@@ -134,21 +144,29 @@ void OCUpdater::backgroundCheckForUpdate()
     }
 }
 
-QString OCUpdater::statusString() const
+QString OCUpdater::statusString(UpdateStatusStringFormat format) const
 {
-    QString updateVersion = _updateInfo.version();
+    QString updateVersion = _updateInfo.versionString();
 
     switch (downloadState()) {
     case Downloading:
-        return tr("Downloading version %1. Please wait …").arg(updateVersion);
+        return tr("Downloading %1. Please wait …").arg(updateVersion);
     case DownloadComplete:
-        return tr("%1 version %2 available. Restart application to start the update.").arg(Theme::instance()->appNameGUI(), updateVersion);
-    case DownloadFailed:
-        return tr("Could not download update. Please click <a href='%1'>here</a> to download the update manually.").arg(_updateInfo.web());
+        return tr("%1 available. Restart application to start the update.").arg(updateVersion);
+    case DownloadFailed: {
+        if (format == UpdateStatusStringFormat::Html) {
+            return tr("Could not download update. Please open <a href='%1'>%1</a> to download the update manually.").arg(_updateInfo.web());
+        }
+        return tr("Could not download update. Please open %1 to download the update manually.").arg(_updateInfo.web());
+    }
     case DownloadTimedOut:
         return tr("Could not check for new updates.");
-    case UpdateOnlyAvailableThroughSystem:
-        return tr("New %1 version %2 is available. Please click <a href='%3'>here</a> to download the update.").arg(Theme::instance()->appNameGUI(), updateVersion, _updateInfo.web());
+    case UpdateOnlyAvailableThroughSystem: {
+        if (format == UpdateStatusStringFormat::Html) {
+            return tr("New %1 is available. Please open <a href='%2'>%2</a> to download the update.").arg(updateVersion, _updateInfo.web());
+        }
+        return tr("New %1 is available. Please open %2 to download the update.").arg(updateVersion, _updateInfo.web());
+    }
     case CheckingServer:
         return tr("Checking update server …");
     case Unknown:
@@ -175,7 +193,7 @@ void OCUpdater::setDownloadState(DownloadState state)
     // or once for system based updates.
     if (_state == OCUpdater::DownloadComplete || (oldState != OCUpdater::UpdateOnlyAvailableThroughSystem
                                                      && _state == OCUpdater::UpdateOnlyAvailableThroughSystem)) {
-        emit newUpdateAvailable(tr("Update Check"), statusString());
+        emit newUpdateAvailable(tr("Update Check"), statusString(), _updateInfo.web());
     }
 }
 
@@ -187,8 +205,31 @@ void OCUpdater::slotStartInstaller()
     settings.setValue(autoUpdateAttemptedC, true);
     settings.sync();
     qCInfo(lcUpdater) << "Running updater" << updateFile;
-    QProcess::startDetached(updateFile, QStringList() << "/S"
-                                                      << "/launch");
+
+    if(updateFile.endsWith(".exe")) {
+        QProcess::startDetached(updateFile, QStringList() << "/S"
+                                                          << "/launch");
+    } else if(updateFile.endsWith(".msi")) {
+        // When MSIs are installed without gui they cannot launch applications
+        // as they lack the user context. That is why we need to run the client
+        // manually here. We wrap the msiexec and client invocation in a powershell
+        // script because owncloud.exe will be shut down for installation.
+        // | Out-Null forces powershell to wait for msiexec to finish.
+        auto preparePathForPowershell = [](QString path) {
+            path.replace("'", "''");
+
+            return QDir::toNativeSeparators(path);
+        };
+
+        QString msiLogFile = cfg.configPath() + "msi.log";
+        QString command = QString("&{msiexec /i '%1' /L*V '%2'| Out-Null ; &'%3'}")
+             .arg(preparePathForPowershell(updateFile))
+             .arg(preparePathForPowershell(msiLogFile))
+             .arg(preparePathForPowershell(QCoreApplication::applicationFilePath()));
+
+        QProcess::startDetached("powershell.exe", QStringList{"-Command", command});
+    }
+    qApp->quit();
 }
 
 void OCUpdater::checkForUpdate()
@@ -219,7 +260,7 @@ bool OCUpdater::updateSucceeded() const
 void OCUpdater::slotVersionInfoArrived()
 {
     _timeoutWatchdog->stop();
-    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
+    auto *reply = qobject_cast<QNetworkReply *>(sender());
     reply->deleteLater();
     if (reply->error() != QNetworkReply::NoError) {
         qCWarning(lcUpdater) << "Failed to reach version check url: " << reply->errorString();
@@ -229,7 +270,7 @@ void OCUpdater::slotVersionInfoArrived()
 
     QString xml = QString::fromUtf8(reply->readAll());
 
-    bool ok;
+    bool ok = false;
     _updateInfo = UpdateInfo::parseString(xml, &ok);
     if (ok) {
         versionInfoArrived(_updateInfo);
@@ -248,21 +289,33 @@ void OCUpdater::slotTimedOut()
 
 NSISUpdater::NSISUpdater(const QUrl &url)
     : OCUpdater(url)
-    , _showFallbackMessage(false)
 {
 }
 
 void NSISUpdater::slotWriteFile()
 {
-    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
+    auto *reply = qobject_cast<QNetworkReply *>(sender());
     if (_file->isOpen()) {
         _file->write(reply->readAll());
     }
 }
 
+void NSISUpdater::wipeUpdateData()
+{
+    ConfigFile cfg;
+    QSettings settings(cfg.configFile(), QSettings::IniFormat);
+    QString updateFileName = settings.value(updateAvailableC).toString();
+    if (!updateFileName.isEmpty())
+        QFile::remove(updateFileName);
+    settings.remove(updateAvailableC);
+    settings.remove(updateTargetVersionC);
+    settings.remove(updateTargetVersionStringC);
+    settings.remove(autoUpdateAttemptedC);
+}
+
 void NSISUpdater::slotDownloadFinished()
 {
-    QNetworkReply *reply = qobject_cast<QNetworkReply *>(sender());
+    auto *reply = qobject_cast<QNetworkReply *>(sender());
     reply->deleteLater();
     if (reply->error() != QNetworkReply::NoError) {
         setDownloadState(DownloadFailed);
@@ -271,12 +324,21 @@ void NSISUpdater::slotDownloadFinished()
 
     QUrl url(reply->url());
     _file->close();
+
+    ConfigFile cfg;
+    QSettings settings(cfg.configFile(), QSettings::IniFormat);
+
+    // remove previously downloaded but not used installer
+    QFile oldTargetFile(settings.value(updateAvailableC).toString());
+    if (oldTargetFile.exists()) {
+        oldTargetFile.remove();
+    }
+
     QFile::copy(_file->fileName(), _targetFile);
     setDownloadState(DownloadComplete);
     qCInfo(lcUpdater) << "Downloaded" << url.toString() << "to" << _targetFile;
-    ConfigFile cfg;
-    QSettings settings(cfg.configFile(), QSettings::IniFormat);
     settings.setValue(updateTargetVersionC, updateInfo().version());
+    settings.setValue(updateTargetVersionStringC, updateInfo().versionString());
     settings.setValue(updateAvailableC, _targetFile);
 }
 
@@ -285,26 +347,36 @@ void NSISUpdater::versionInfoArrived(const UpdateInfo &info)
     ConfigFile cfg;
     QSettings settings(cfg.configFile(), QSettings::IniFormat);
     qint64 infoVersion = Helper::stringVersionToInt(info.version());
-    qint64 seenVersion = Helper::stringVersionToInt(settings.value(seenVersionC).toString());
+    auto seenString = settings.value(seenVersionC).toString();
+    qint64 seenVersion = Helper::stringVersionToInt(seenString);
     qint64 currVersion = Helper::currentVersionToInt();
-    if (info.version().isEmpty()
-        || infoVersion <= currVersion
-        || infoVersion <= seenVersion) {
+    qCInfo(lcUpdater) << "Version info arrived:"
+            << "Your version:" << currVersion
+            << "Skipped version:" << seenVersion << seenString
+            << "Available version:" << infoVersion << info.version()
+            << "Available version string:" << info.versionString()
+            << "Web url:" << info.web()
+            << "Download url:" << info.downloadUrl();
+    if (info.version().isEmpty())
+    {
+        qCInfo(lcUpdater) << "No version information available at the moment";
+        setDownloadState(UpToDate);
+    } else if (infoVersion <= currVersion
+               || infoVersion <= seenVersion) {
         qCInfo(lcUpdater) << "Client is on latest version!";
         setDownloadState(UpToDate);
     } else {
         QString url = info.downloadUrl();
-        qint64 autoUpdateFailedVersion =
-            Helper::stringVersionToInt(settings.value(autoUpdateFailedVersionC).toString());
-        if (url.isEmpty() || _showFallbackMessage || infoVersion == autoUpdateFailedVersion) {
-            showDialog(info);
-        }
-        if (!url.isEmpty()) {
-            _targetFile = cfg.configPath() + url.mid(url.lastIndexOf('/'));
+        if (url.isEmpty()) {
+            showNoUrlDialog(info);
+        } else {
+            _targetFile = cfg.configPath() + url.mid(url.lastIndexOf('/')+1);
             if (QFile(_targetFile).exists()) {
                 setDownloadState(DownloadComplete);
             } else {
-                QNetworkReply *reply = qnam()->get(QNetworkRequest(QUrl(url)));
+                auto request = QNetworkRequest(QUrl(url));
+                request.setAttribute(QNetworkRequest::RedirectPolicyAttribute, QNetworkRequest::NoLessSafeRedirectPolicy);
+                QNetworkReply *reply = qnam()->get(request);
                 connect(reply, &QIODevice::readyRead, this, &NSISUpdater::slotWriteFile);
                 connect(reply, &QNetworkReply::finished, this, &NSISUpdater::slotDownloadFinished);
                 setDownloadState(Downloading);
@@ -316,27 +388,28 @@ void NSISUpdater::versionInfoArrived(const UpdateInfo &info)
     }
 }
 
-void NSISUpdater::showDialog(const UpdateInfo &info)
+void NSISUpdater::showNoUrlDialog(const UpdateInfo &info)
 {
     // if the version tag is set, there is a newer version.
-    QDialog *msgBox = new QDialog;
+    auto *msgBox = new QDialog;
     msgBox->setAttribute(Qt::WA_DeleteOnClose);
+    msgBox->setWindowFlags(msgBox->windowFlags() & ~Qt::WindowContextHelpButtonHint);
 
-    QIcon infoIcon = msgBox->style()->standardIcon(QStyle::SP_MessageBoxInformation, nullptr, nullptr);
-    int iconSize = msgBox->style()->pixelMetric(QStyle::PM_MessageBoxIconSize, nullptr, nullptr);
+    QIcon infoIcon = msgBox->style()->standardIcon(QStyle::SP_MessageBoxInformation);
+    int iconSize = msgBox->style()->pixelMetric(QStyle::PM_MessageBoxIconSize);
 
     msgBox->setWindowIcon(infoIcon);
 
-    QVBoxLayout *layout = new QVBoxLayout(msgBox);
-    QHBoxLayout *hlayout = new QHBoxLayout;
+    auto *layout = new QVBoxLayout(msgBox);
+    auto *hlayout = new QHBoxLayout;
     layout->addLayout(hlayout);
 
     msgBox->setWindowTitle(tr("New Version Available"));
 
-    QLabel *ico = new QLabel;
+    auto *ico = new QLabel;
     ico->setFixedSize(iconSize, iconSize);
     ico->setPixmap(infoIcon.pixmap(iconSize));
-    QLabel *lbl = new QLabel;
+    auto *lbl = new QLabel;
     QString txt = tr("<p>A new version of the %1 Client is available.</p>"
                      "<p><b>%2</b> is available for download. The installed version is %3.</p>")
                       .arg(Utility::escape(Theme::instance()->appNameGUI()),
@@ -349,8 +422,7 @@ void NSISUpdater::showDialog(const UpdateInfo &info)
     hlayout->addWidget(ico);
     hlayout->addWidget(lbl);
 
-    QDialogButtonBox *bb = new QDialogButtonBox;
-    bb->setWindowFlags(bb->windowFlags() & ~Qt::WindowContextHelpButtonHint);
+    auto *bb = new QDialogButtonBox;
     QPushButton *skip = bb->addButton(tr("Skip this version"), QDialogButtonBox::ResetRole);
     QPushButton *reject = bb->addButton(tr("Skip this time"), QDialogButtonBox::AcceptRole);
     QPushButton *getupdate = bb->addButton(tr("Get update"), QDialogButtonBox::AcceptRole);
@@ -360,60 +432,103 @@ void NSISUpdater::showDialog(const UpdateInfo &info)
     connect(getupdate, &QAbstractButton::clicked, msgBox, &QDialog::accept);
 
     connect(skip, &QAbstractButton::clicked, this, &NSISUpdater::slotSetSeenVersion);
-    connect(getupdate, SIGNAL(clicked()), SLOT(slotOpenUpdateUrl()));
+    connect(getupdate, &QAbstractButton::clicked, this, &NSISUpdater::slotOpenUpdateUrl);
 
     layout->addWidget(bb);
 
     msgBox->open();
 }
 
-NSISUpdater::UpdateState NSISUpdater::updateStateOnStart()
+void NSISUpdater::showUpdateErrorDialog(const QString &targetVersion)
+{
+    auto msgBox = new QDialog;
+    msgBox->setAttribute(Qt::WA_DeleteOnClose);
+    msgBox->setWindowFlags(msgBox->windowFlags() & ~Qt::WindowContextHelpButtonHint);
+
+    QIcon infoIcon = msgBox->style()->standardIcon(QStyle::SP_MessageBoxInformation);
+    int iconSize = msgBox->style()->pixelMetric(QStyle::PM_MessageBoxIconSize);
+
+    msgBox->setWindowIcon(infoIcon);
+
+    auto layout = new QVBoxLayout(msgBox);
+    auto hlayout = new QHBoxLayout;
+    layout->addLayout(hlayout);
+
+    msgBox->setWindowTitle(tr("Update Failed"));
+
+    auto ico = new QLabel;
+    ico->setFixedSize(iconSize, iconSize);
+    ico->setPixmap(infoIcon.pixmap(iconSize));
+    auto lbl = new QLabel;
+    QString txt = tr("<p>A new version of the %1 Client is available but the updating process failed.</p>"
+                     "<p><b>%2</b> has been downloaded. The installed version is %3. If you confirm restart and update, your computer may reboot to complete the installation.</p>")
+                      .arg(Utility::escape(Theme::instance()->appNameGUI()),
+                          Utility::escape(targetVersion), Utility::escape(clientVersion()));
+
+    lbl->setText(txt);
+    lbl->setTextFormat(Qt::RichText);
+    lbl->setWordWrap(true);
+
+    hlayout->addWidget(ico);
+    hlayout->addWidget(lbl);
+
+    auto bb = new QDialogButtonBox;
+    auto skip = bb->addButton(tr("Skip this version"), QDialogButtonBox::ResetRole);
+    auto askagain = bb->addButton(tr("Ask again later"), QDialogButtonBox::ResetRole);
+    auto retry = bb->addButton(tr("Restart and update"), QDialogButtonBox::AcceptRole);
+    auto getupdate = bb->addButton(tr("Update manually"), QDialogButtonBox::AcceptRole);
+
+    connect(skip, &QAbstractButton::clicked, msgBox, &QDialog::reject);
+    connect(askagain, &QAbstractButton::clicked, msgBox, &QDialog::reject);
+    connect(retry, &QAbstractButton::clicked, msgBox, &QDialog::accept);
+    connect(getupdate, &QAbstractButton::clicked, msgBox, &QDialog::accept);
+
+    connect(skip, &QAbstractButton::clicked, this, [this]() {
+        wipeUpdateData();
+        slotSetSeenVersion();
+    });
+    // askagain: do nothing
+    connect(retry, &QAbstractButton::clicked, this, [this]() {
+        slotStartInstaller();
+    });
+    connect(getupdate, &QAbstractButton::clicked, this, [this]() {
+        slotOpenUpdateUrl();
+    });
+
+    layout->addWidget(bb);
+
+    msgBox->open();
+}
+
+bool NSISUpdater::handleStartup()
 {
     ConfigFile cfg;
     QSettings settings(cfg.configFile(), QSettings::IniFormat);
     QString updateFileName = settings.value(updateAvailableC).toString();
     // has the previous run downloaded an update?
     if (!updateFileName.isEmpty() && QFile(updateFileName).exists()) {
+        qCInfo(lcUpdater) << "An updater file is available";
         // did it try to execute the update?
         if (settings.value(autoUpdateAttemptedC, false).toBool()) {
-            // clean up
-            settings.remove(autoUpdateAttemptedC);
-            settings.remove(updateAvailableC);
-            QFile::remove(updateFileName);
             if (updateSucceeded()) {
-                // success: clean up even more
-                settings.remove(updateTargetVersionC);
-                settings.remove(autoUpdateFailedVersionC);
-                return NoUpdate;
+                // success: clean up
+                qCInfo(lcUpdater) << "The requested update attempt has succeeded"
+                        << Helper::currentVersionToInt();
+                wipeUpdateData();
+                return false;
             } else {
-                // auto update failed. Set autoUpdateFailedVersion as a hint
-                // for visual fallback notification
-                QString targetVersion = settings.value(updateTargetVersionC).toString();
-                settings.setValue(autoUpdateFailedVersionC, targetVersion);
-                settings.remove(updateTargetVersionC);
-                return UpdateFailed;
+                // auto update failed. Ask user what to do
+                qCInfo(lcUpdater) << "The requested update attempt has failed"
+                        << settings.value(updateTargetVersionC).toString();
+                showUpdateErrorDialog(settings.value(updateTargetVersionStringC).toString());
+                return false;
             }
         } else {
-            if (!settings.contains(autoUpdateFailedVersionC)) {
-                return UpdateAvailable;
-            }
+            qCInfo(lcUpdater) << "Triggering an update";
+            return performUpdate();
         }
     }
-    return NoUpdate;
-}
-
-bool NSISUpdater::handleStartup()
-{
-    switch (updateStateOnStart()) {
-    case NSISUpdater::UpdateAvailable:
-        return performUpdate();
-    case NSISUpdater::UpdateFailed:
-        _showFallbackMessage = true;
-        return false;
-    case NSISUpdater::NoUpdate:
-    default:
-        return false;
-    }
+    return false;
 }
 
 void NSISUpdater::slotSetSeenVersion()
